@@ -28,8 +28,9 @@ from os_win.utils.compute import vmutils
 from os_win.utils import hostutils
 from os_win.utils.network import networkutils
 from os_win.utils import pathutils
+from os_win.utils.storage import diskutils
 from os_win.utils.storage.initiator import iscsi_cli_utils
-from os_win.utils.storage.initiator import iscsi_wmi_utils
+from os_win.utils.storage.initiator import iscsi_utils
 from os_win.utils.storage import smbutils
 from os_win.utils.storage.virtdisk import vhdutils
 from os_win import utilsfactory
@@ -94,9 +95,11 @@ class TestHyperVUtilsFactory(test_base.OsWinBaseTestCase):
         self._check_get_class(expected_class=rdpconsoleutils.RDPConsoleUtils,
                               class_type='rdpconsoleutils')
 
+    @mock.patch.object(iscsi_utils.ISCSIInitiatorUtils, '__init__',
+                       lambda *args, **kwargs: None)
     def test_get_iscsi_initiator_utils(self):
         self._test_get_initiator_utils(
-            expected_class=iscsi_wmi_utils.ISCSIInitiatorWMIUtils)
+            expected_class=iscsi_utils.ISCSIInitiatorUtils)
 
     def test_get_iscsi_initiator_utils_force_v1(self):
         self._test_get_initiator_utils(
@@ -117,3 +120,8 @@ class TestHyperVUtilsFactory(test_base.OsWinBaseTestCase):
         self._check_get_class(
             expected_class=type(mock_cls_fcutils.return_value),
             class_type='fc_utils')
+
+    def test_get_diskutils(self):
+        self._check_get_class(
+            expected_class=diskutils.DiskUtils,
+            class_type='diskutils')
