@@ -96,7 +96,8 @@ class JobUtilsTestCase(base.BaseTestCase):
         job = self.jobutils._wait_for_job(self._FAKE_JOB_PATH)
         self.assertEqual(mock_job, job)
 
-    def test_stop_jobs(self):
+    @mock.patch.object(jobutils, 'wmi', create=True)
+    def test_stop_jobs(self, mock_wmi):
         mock_job1 = mock.MagicMock(Cancellable=True)
         mock_job2 = mock.MagicMock(Cancellable=True)
         mock_job3 = mock.MagicMock(Cancellable=True)
@@ -104,9 +105,11 @@ class JobUtilsTestCase(base.BaseTestCase):
         mock_job2.JobState = 3
         mock_job3.JobState = constants.JOB_STATE_KILLED
 
+        mock_wmi.WMI.side_effect = [mock_job1, mock_job2, mock_job3]
+
         mock_vm = mock.MagicMock()
         mock_vm_jobs = [mock_job1, mock_job2, mock_job3]
-        mock_vm.associators.return_value = mock_vm_jobs
+        self.jobutils._conn.Msvm_AffectedJobElement.return_value = mock_vm_jobs
 
         self.jobutils.stop_jobs(mock_vm)
 
