@@ -35,6 +35,7 @@ from six.moves import range  # noqa
 from os_win._i18n import _, _LW
 from os_win import constants
 from os_win import exceptions
+from os_win.utils import _wqlutils
 from os_win.utils import jobutils
 from os_win.utils.metrics import metricsutils
 from os_win.utils import pathutils
@@ -65,9 +66,18 @@ class VMUtils(object):
     _SYNTHETIC_ETHERNET_PORT_SETTING_DATA_CLASS = (
         'Msvm_SyntheticEthernetPortSettingData')
     _AFFECTED_JOB_ELEMENT_CLASS = "Msvm_AffectedJobElement"
+    _CIM_RES_ALLOC_SETTING_DATA_CLASS = 'Cim_ResourceAllocationSettingData'
     _COMPUTER_SYSTEM_CLASS = "Msvm_ComputerSystem"
     _LOGICAL_IDENTITY_CLASS = 'Msvm_LogicalIdentity'
 
+    _S3_DISP_CTRL_RES_SUB_TYPE = 'Microsoft:Hyper-V:S3 Display Controller'
+    _SYNTH_DISP_CTRL_RES_SUB_TYPE = ('Microsoft:Hyper-V:Synthetic Display '
+                                     'Controller')
+    _SYNTH_3D_DISP_CTRL_RES_SUB_TYPE = ('Microsoft:Hyper-V:Synthetic 3D '
+                                        'Display Controller')
+    _SYNTH_3D_DISP_ALLOCATION_SETTING_DATA_CLASS = (
+        'Msvm_Synthetic3DDisplayControllerSettingData')
+
     _VIRTUAL_SYSTEM_SUBTYPE = 'VirtualSystemSubType'
     _VIRTUAL_SYSTEM_TYPE_REALIZED = 'Microsoft:Hyper-V:System:Realized'
     _VIRTUAL_SYSTEM_SUBTYPE_GEN2 = 'Microsoft:Hyper-V:SubType:2'
@@ -80,6 +90,26 @@ class VMUtils(object):
     _VIRTUAL_SYSTEM_CURRENT_SETTINGS = 3
     _AUTOMATIC_STARTUP_ACTION_NONE = 2
 
+    _remote_fx_res_map = {
+        constants.REMOTEFX_MAX_RES_1024x768: 0,
+        constants.REMOTEFX_MAX_RES_1280x1024: 1,
+        constants.REMOTEFX_MAX_RES_1600x1200: 2,
+        constants.REMOTEFX_MAX_RES_1920x1200: 3,
+        constants.REMOTEFX_MAX_RES_2560x1600: 4
+    }
+
+    _remotefx_max_monitors_map = {
+        # defines the maximum number of monitors for a given
+        # resolution
+        constants.REMOTEFX_MAX_RES_1024x768: 4,
+        constants.REMOTEFX_MAX_RES_1280x1024: 4,
+        constants.REMOTEFX_MAX_RES_1600x1200: 3,
+        constants.REMOTEFX_MAX_RES_1920x1200: 2,
+        constants.REMOTEFX_MAX_RES_2560x1600: 1
+    }
+
+    _DISP_CTRL_ADDRESS_DX_11 = "02C1,00000000,01"
+
     _vm_power_states_map = {constants.HYPERV_VM_STATE_ENABLED: 2,
                             constants.HYPERV_VM_STATE_DISABLED: 3,
                             constants.HYPERV_VM_STATE_SHUTTING_DOWN: 4,
@@ -127,12 +157,9 @@ class VMUtils(object):
                     VirtualSystemType=self._VIRTUAL_SYSTEM_TYPE_REALIZED)]
 
     def get_vm_summary_info(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
 
-        vmsettings = vm.associators(
-            wmi_association_class=self._SETTINGS_DEFINE_STATE_CLASS,
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        settings_paths = [v.path_() for v in vmsettings]
+        settings_paths = [vmsettings.path_()]
         # See http://msdn.microsoft.com/en-us/library/cc160706%28VS.85%29.aspx
         (ret_val, summary_info) = self._vs_man_svc.GetSummaryInformation(
             [constants.VM_SUMMARY_NUM_PROCS,
@@ -170,15 +197,20 @@ class VMUtils(object):
         settings = self.get_vm_summary_info(vm_name)
         return settings['EnabledState']
 
-    def _lookup_vm_check(self, vm_name):
+    def _lookup_vm_check(self, vm_name, as_vssd=True):
 
-        vm = self._lookup_vm(vm_name)
+        vm = self._lookup_vm(vm_name, as_vssd)
         if not vm:
             raise exceptions.HyperVVMNotFoundException(vm_name=vm_name)
         return vm
 
-    def _lookup_vm(self, vm_name):
-        vms = self._conn.Msvm_ComputerSystem(ElementName=vm_name)
+    def _lookup_vm(self, vm_name, as_vssd=True):
+        if as_vssd:
+            vms = self._conn.Msvm_VirtualSystemSettingData(
+                ElementName=vm_name,
+                VirtualSystemType=self._VIRTUAL_SYSTEM_TYPE_REALIZED)
+        else:
+            vms = self._conn.Msvm_ComputerSystem(ElementName=vm_name)
         n = len(vms)
         if n == 0:
             return None
@@ -192,20 +224,14 @@ class VMUtils(object):
         return self._lookup_vm(vm_name) is not None
 
     def get_vm_id(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
         return vm.Name
 
-    def _get_vm_setting_data(self, vm):
-        vmsettings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        # Avoid snapshots
-        return [s for s in vmsettings if
-                s.VirtualSystemType == self._VIRTUAL_SYSTEM_TYPE_REALIZED][0]
-
     def _set_vm_memory(self, vmsetting, memory_mb, memory_per_numa_node,
                        dynamic_memory_ratio):
-        mem_settings = vmsetting.associators(
-            wmi_result_class=self._MEMORY_SETTING_DATA_CLASS)[0]
+        mem_settings = _wqlutils.get_element_associated_class(
+            self._conn, self._MEMORY_SETTING_DATA_CLASS,
+            element_instance_id=vmsetting.InstanceID)[0]
 
         max_mem = int(memory_mb)
         mem_settings.Limit = max_mem
@@ -232,8 +258,10 @@ class VMUtils(object):
 
     def _set_vm_vcpus(self, vmsetting, vcpus_num, vcpus_per_numa_node,
                       limit_cpu_features):
-        procsetting = vmsetting.associators(
-            wmi_result_class=self._PROCESSOR_SETTING_DATA_CLASS)[0]
+        procsetting = _wqlutils.get_element_associated_class(
+            self._conn, self._PROCESSOR_SETTING_DATA_CLASS,
+            element_instance_id=vmsetting.InstanceID)[0]
+
         vcpus = int(vcpus_num)
         procsetting.VirtualQuantity = vcpus
         procsetting.Reservation = vcpus
@@ -247,8 +275,7 @@ class VMUtils(object):
 
     def update_vm(self, vm_name, memory_mb, memory_per_numa_node, vcpus_num,
                   vcpus_per_numa_node, limit_cpu_features, dynamic_mem_ratio):
-        vm = self._lookup_vm_check(vm_name)
-        vmsetting = self._get_vm_setting_data(vm)
+        vmsetting = self._lookup_vm_check(vm_name)
         self._set_vm_memory(vmsetting, memory_mb, memory_per_numa_node,
                             dynamic_mem_ratio)
         self._set_vm_vcpus(vmsetting, vcpus_num, vcpus_per_numa_node,
@@ -283,10 +310,10 @@ class VMUtils(object):
         # vNUMA and dynamic memory are mutually exclusive
         vnuma_enabled = False if dynamic_memory_ratio > 1 else True
 
-        vm = self._create_vm_obj(vm_name, vnuma_enabled, vm_gen, notes,
-                                 instance_path)
+        self._create_vm_obj(vm_name, vnuma_enabled, vm_gen, notes,
+                            instance_path)
 
-        vmsetting = self._get_vm_setting_data(vm)
+        vmsetting = self._lookup_vm_check(vm_name)
 
         LOG.debug('Setting memory for vm %s', vm_name)
         self._set_vm_memory(vmsetting, memory_mb, None, dynamic_memory_ratio)
@@ -321,10 +348,7 @@ class VMUtils(object):
          ret_val) = self._vs_man_svc.DefineSystem(
             ResourceSettings=[], ReferenceConfiguration=None,
             SystemSettings=vs_data.GetText_(1))
-        job = self._jobutils.check_ret_val(ret_val, job_path)
-        if not vm_path and job:
-            vm_path = job.associators(self._AFFECTED_JOB_ELEMENT_CLASS)[0]
-        return self._get_wmi_obj(vm_path)
+        self._jobutils.check_ret_val(ret_val, job_path)
 
     def _modify_virtual_system(self, vmsetting):
         (job_path, ret_val) = self._vs_man_svc.ModifySystemSettings(
@@ -332,23 +356,21 @@ class VMUtils(object):
         self._jobutils.check_ret_val(ret_val, job_path)
 
     def get_vm_scsi_controller(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        return self._get_vm_scsi_controller(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
+        return self._get_vm_scsi_controller(vmsettings)
 
-    def _get_vm_scsi_controller(self, vm):
-        vmsettings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        rasds = vmsettings[0].associators(
-            wmi_result_class=self._RESOURCE_ALLOC_SETTING_DATA_CLASS)
+    def _get_vm_scsi_controller(self, vmsettings):
+        rasds = _wqlutils.get_element_associated_class(
+            self._conn, self._RESOURCE_ALLOC_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)
         res = [r for r in rasds
                if r.ResourceSubType == self._SCSI_CTRL_RES_SUB_TYPE][0]
         return res.path_()
 
-    def _get_vm_ide_controller(self, vm, ctrller_addr):
-        vmsettings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        rasds = vmsettings[0].associators(
-            wmi_result_class=self._RESOURCE_ALLOC_SETTING_DATA_CLASS)
+    def _get_vm_ide_controller(self, vmsettings, ctrller_addr):
+        rasds = _wqlutils.get_element_associated_class(
+            self._conn, self._RESOURCE_ALLOC_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)
         ide_ctrls = [r for r in rasds
                      if r.ResourceSubType == self._IDE_CTRL_RES_SUB_TYPE
                      and r.Address == str(ctrller_addr)]
@@ -356,8 +378,8 @@ class VMUtils(object):
         return ide_ctrls[0].path_() if ide_ctrls else None
 
     def get_vm_ide_controller(self, vm_name, ctrller_addr):
-        vm = self._lookup_vm_check(vm_name)
-        return self._get_vm_ide_controller(vm, ctrller_addr)
+        vmsettings = self._lookup_vm_check(vm_name)
+        return self._get_vm_ide_controller(vmsettings, ctrller_addr)
 
     def get_attached_disks(self, scsi_controller_path):
         volumes = self._conn.query(
@@ -394,22 +416,22 @@ class VMUtils(object):
         return obj
 
     def attach_scsi_drive(self, vm_name, path, drive_type=constants.DISK):
-        vm = self._lookup_vm_check(vm_name)
-        ctrller_path = self._get_vm_scsi_controller(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
+        ctrller_path = self._get_vm_scsi_controller(vmsettings)
         drive_addr = self.get_free_controller_slot(ctrller_path)
         self.attach_drive(vm_name, path, ctrller_path, drive_addr, drive_type)
 
     def attach_ide_drive(self, vm_name, path, ctrller_addr, drive_addr,
                          drive_type=constants.DISK):
-        vm = self._lookup_vm_check(vm_name)
-        ctrller_path = self._get_vm_ide_controller(vm, ctrller_addr)
+        vmsettings = self._lookup_vm_check(vm_name)
+        ctrller_path = self._get_vm_ide_controller(vmsettings, ctrller_addr)
         self.attach_drive(vm_name, path, ctrller_path, drive_addr, drive_type)
 
     def attach_drive(self, vm_name, path, ctrller_path, drive_addr,
                      drive_type=constants.DISK):
         """Create a drive and attach it to the vm."""
 
-        vm = self._lookup_vm_check(vm_name)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
 
         if drive_type == constants.DISK:
             res_sub_type = self._DISK_DRIVE_RES_SUB_TYPE
@@ -443,18 +465,18 @@ class VMUtils(object):
     def create_scsi_controller(self, vm_name):
         """Create an iscsi controller ready to mount volumes."""
 
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
         scsicontrl = self._get_new_resource_setting_data(
             self._SCSI_CTRL_RES_SUB_TYPE)
 
         scsicontrl.VirtualSystemIdentifiers = ['{' + str(uuid.uuid4()) + '}']
-        self._jobutils.add_virt_resource(scsicontrl, vm)
+        self._jobutils.add_virt_resource(scsicontrl, vmsettings)
 
     def attach_volume_to_controller(self, vm_name, controller_path, address,
                                     mounted_disk_path, serial=None):
         """Attach a volume to a controller."""
 
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
 
         diskdrive = self._get_new_resource_setting_data(
             self._PHYS_DISK_RES_SUB_TYPE)
@@ -463,7 +485,8 @@ class VMUtils(object):
         diskdrive.Parent = controller_path
         diskdrive.HostResource = [mounted_disk_path]
 
-        diskdrive_path = self._jobutils.add_virt_resource(diskdrive, vm)[0]
+        diskdrive_path = self._jobutils.add_virt_resource(diskdrive,
+                                                          vmsettings)[0]
 
         if serial:
             # Apparently this can't be set when the resource is added.
@@ -493,8 +516,8 @@ class VMUtils(object):
         # TODO(lpetrut): remove this method after the patch fixing
         # swapped disks after host reboot merges in Nova.
         disk_found = False
-        vm = self._lookup_vm_check(vm_name)
-        (disk_resources, volume_resources) = self._get_vm_disks(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
+        (disk_resources, volume_resources) = self._get_vm_disks(vmsettings)
         for disk_resource in disk_resources + volume_resources:
             if (disk_resource.Parent == controller_path and
                     self._get_disk_resource_address(disk_resource) ==
@@ -533,9 +556,9 @@ class VMUtils(object):
         new_nic_data.VirtualSystemIdentifiers = ['{' + str(uuid.uuid4()) + '}']
 
         # Add the new nic to the vm
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
 
-        self._jobutils.add_virt_resource(new_nic_data, vm)
+        self._jobutils.add_virt_resource(new_nic_data, vmsettings)
 
     def destroy_nic(self, vm_name, nic_name):
         """Destroys the NIC with the given nic_name from the given VM.
@@ -548,9 +571,9 @@ class VMUtils(object):
         self._jobutils.remove_virt_resource(nic_data)
 
     def soft_shutdown_vm(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        shutdown_component = vm.associators(
-            wmi_result_class=self._SHUTDOWN_COMPONENT)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
+        shutdown_component = self._conn.Msvm_ShutdownComponent(
+            SystemName=vm.Name)
 
         if not shutdown_component:
             # If no shutdown_component is found, it means the VM is already
@@ -564,7 +587,7 @@ class VMUtils(object):
 
     def set_vm_state(self, vm_name, req_state):
         """Set the desired state of the VM."""
-        vm = self._lookup_vm_check(vm_name)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
         (job_path,
          ret_val) = vm.RequestStateChange(self._vm_power_states_map[req_state])
         # Invalid state for current operation (32775) typically means that
@@ -578,8 +601,8 @@ class VMUtils(object):
         return disk_resource.HostResource
 
     def get_vm_storage_paths(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        (disk_resources, volume_resources) = self._get_vm_disks(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
+        (disk_resources, volume_resources) = self._get_vm_disks(vmsettings)
 
         volume_drives = []
         for volume_resource in volume_resources:
@@ -594,14 +617,13 @@ class VMUtils(object):
         return (disk_files, volume_drives)
 
     def get_vm_disks(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        return self._get_vm_disks(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
+        return self._get_vm_disks(vmsettings)
 
-    def _get_vm_disks(self, vm):
-        vmsettings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        rasds = vmsettings[0].associators(
-            wmi_result_class=self._STORAGE_ALLOC_SETTING_DATA_CLASS)
+    def _get_vm_disks(self, vmsettings):
+        rasds = _wqlutils.get_element_associated_class(
+            self._conn, self._STORAGE_ALLOC_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)
         disk_resources = [r for r in rasds if
                           r.ResourceSubType in
                           [self._HARD_DISK_RES_SUB_TYPE,
@@ -609,8 +631,9 @@ class VMUtils(object):
 
         if (self._RESOURCE_ALLOC_SETTING_DATA_CLASS !=
                 self._STORAGE_ALLOC_SETTING_DATA_CLASS):
-            rasds = vmsettings[0].associators(
-                wmi_result_class=self._RESOURCE_ALLOC_SETTING_DATA_CLASS)
+            rasds = _wqlutils.get_element_associated_class(
+                self._conn, self._RESOURCE_ALLOC_SETTING_DATA_CLASS,
+                element_instance_id=vmsettings.InstanceID)
 
         volume_resources = [r for r in rasds if
                             r.ResourceSubType == self._PHYS_DISK_RES_SUB_TYPE]
@@ -618,7 +641,7 @@ class VMUtils(object):
         return (disk_resources, volume_resources)
 
     def destroy_vm(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
 
         # Remove the VM. It does not destroy any associated virtual disk.
         (job_path, ret_val) = self._vs_man_svc.DestroySystem(vm.path_())
@@ -628,7 +651,7 @@ class VMUtils(object):
         return wmi.WMI(moniker=path.replace('\\', '/'))
 
     def take_vm_snapshot(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
         vs_snap_svc = self._conn.Msvm_VirtualSystemSnapshotService()[0]
 
         (job_path, snp_setting_data, ret_val) = vs_snap_svc.CreateSnapshot(
@@ -636,11 +659,10 @@ class VMUtils(object):
             SnapshotType=self._SNAPSHOT_FULL)
         self._jobutils.check_ret_val(ret_val, job_path)
 
-        job = self._get_wmi_obj(job_path)
-        snp_setting_data = job.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)[0]
+        snp_setting_data_path = self._conn.Msvm_MostCurrentSnapshotInBranch(
+            Antecedent=vm.path_())[0].Dependent
 
-        return snp_setting_data.path_()
+        return snp_setting_data_path
 
     def remove_vm_snapshot(self, snapshot_path):
         vs_snap_svc = self._conn.Msvm_VirtualSystemSnapshotService()[0]
@@ -654,12 +676,11 @@ class VMUtils(object):
         self._metricsutils.enable_vm_metrics_collection(vm_name)
 
     def get_vm_dvd_disk_paths(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
 
-        settings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)[0]
-        sasds = settings.associators(
-            wmi_result_class=self._STORAGE_ALLOC_SETTING_DATA_CLASS)
+        sasds = _wqlutils.get_element_associated_class(
+            self._conn, self._STORAGE_ALLOC_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)
 
         dvd_paths = [sasd.HostResource[0] for sasd in sasds
                      if sasd.ResourceSubType == self._DVD_DISK_RES_SUB_TYPE]
@@ -744,12 +765,11 @@ class VMUtils(object):
     def get_vm_serial_port_connection(self, vm_name, update_connection=None):
         # TODO(lpetrut): Remove this method after the patch implementing
         # serial console access support merges in Nova.
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
 
-        vmsettings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        rasds = vmsettings[0].associators(
-            wmi_result_class=self._SERIAL_PORT_SETTING_DATA_CLASS)
+        rasds = _wqlutils.get_element_associated_class(
+            self._conn, self._SERIAL_PORT_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)
         serial_port = (
             [r for r in rasds if
              r.ResourceSubType == self._SERIAL_PORT_RES_SUB_TYPE][0])
@@ -761,11 +781,10 @@ class VMUtils(object):
         if len(serial_port.Connection) > 0:
             return serial_port.Connection[0]
 
-    def _get_vm_serial_ports(self, vm):
-        vmsettings = vm.associators(
-            wmi_result_class=self._VIRTUAL_SYSTEM_SETTING_DATA_CLASS)
-        rasds = vmsettings[0].associators(
-            wmi_result_class=self._SERIAL_PORT_SETTING_DATA_CLASS)
+    def _get_vm_serial_ports(self, vmsettings):
+        rasds = _wqlutils.get_element_associated_class(
+            self._conn, self._SERIAL_PORT_SETTING_DATA_CLASS,
+            element_instance_id=vmsettings.InstanceID)
         serial_ports = (
             [r for r in rasds if
              r.ResourceSubType == self._SERIAL_PORT_RES_SUB_TYPE]
@@ -773,16 +792,16 @@ class VMUtils(object):
         return serial_ports
 
     def set_vm_serial_port_connection(self, vm_name, port_number, pipe_path):
-        vm = self._lookup_vm_check(vm_name)
+        vmsettings = self._lookup_vm_check(vm_name)
 
-        serial_port = self._get_vm_serial_ports(vm)[port_number - 1]
+        serial_port = self._get_vm_serial_ports(vmsettings)[port_number - 1]
         serial_port.Connection = [pipe_path]
 
         self._jobutils.modify_virt_resource(serial_port)
 
     def get_vm_serial_port_connections(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        serial_ports = self._get_vm_serial_ports(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
+        serial_ports = self._get_vm_serial_ports(vmsettings)
         conns = [serial_port.Connection[0]
                  for serial_port in serial_ports
                  if serial_port.Connection and serial_port.Connection[0]]
@@ -791,7 +810,7 @@ class VMUtils(object):
     def get_active_instances(self):
         """Return the names of all the active instances known to Hyper-V."""
         vm_names = self.list_instances()
-        vms = [self._lookup_vm(vm_name) for vm_name in vm_names]
+        vms = [self._lookup_vm(vm_name, as_vssd=False) for vm_name in vm_names]
         active_vm_names = [v.ElementName for v in vms
             if v.EnabledState == constants.HYPERV_VM_STATE_ENABLED]
 
@@ -834,8 +853,7 @@ class VMUtils(object):
         return query
 
     def _get_instance_notes(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        vmsettings = self._get_vm_setting_data(vm)
+        vmsettings = self._lookup_vm_check(vm_name)
         return [note for note in vmsettings.Notes if note]
 
     def get_instance_uuid(self, vm_name):
@@ -848,15 +866,14 @@ class VMUtils(object):
                                             constants.HYPERV_VM_STATE_OTHER)
 
     def get_vm_generation(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
-        vssd = self._get_vm_setting_data(vm)
+        vssd = self._lookup_vm_check(vm_name)
         if hasattr(vssd, self._VIRTUAL_SYSTEM_SUBTYPE):
             # expected format: 'Microsoft:Hyper-V:SubType:2'
             return int(vssd.VirtualSystemSubType.split(':')[-1])
         return constants.VM_GEN_1
 
     def stop_vm_jobs(self, vm_name):
-        vm = self._lookup_vm_check(vm_name)
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
         self._jobutils.stop_jobs(vm)
 
     def enable_secure_boot(self, vm_name, msft_ca_required):
@@ -869,8 +886,7 @@ class VMUtils(object):
                                  Authority for Secure Boot. Only Linux
                                  guests require this CA.
         """
-        vm = self._lookup_vm_check(vm_name)
-        vs_data = self._get_vm_setting_data(vm)
+        vs_data = self._lookup_vm_check(vm_name)
         self._set_secure_boot(vs_data, msft_ca_required)
         self._modify_virtual_system(vs_data)
 
@@ -907,14 +923,11 @@ class VMUtils(object):
         drive = self._get_mounted_disk_resource_from_path(
             drive_path, is_physical=is_physical)
 
-        if is_physical:
-            bssd = drive.associators(
-                wmi_association_class=self._LOGICAL_IDENTITY_CLASS)[0]
-        else:
-            rasd = wmi.WMI(moniker=drive.Parent)
-            bssd = rasd.associators(
-                wmi_association_class=self._LOGICAL_IDENTITY_CLASS)[0]
-        return bssd
+        rasd_path = drive.path_() if is_physical else drive.Parent
+        bssd_path = self._conn.Msvm_LogicalIdentity(
+            SystemElement=rasd_path)[0].SameElement
+
+        return bssd_path
 
     def set_boot_order(self, vm_name, device_boot_order):
         if self.get_vm_generation(vm_name) == constants.VM_GEN_1:
@@ -923,21 +936,93 @@ class VMUtils(object):
             self._set_boot_order_gen2(vm_name, device_boot_order)
 
     def _set_boot_order_gen1(self, vm_name, device_boot_order):
-        vm = self._lookup_vm_check(vm_name)
-
-        vssd = self._get_vm_setting_data(vm)
+        vssd = self._lookup_vm_check(vm_name)
         vssd.BootOrder = tuple(device_boot_order)
 
         self._modify_virtual_system(vssd)
 
     def _set_boot_order_gen2(self, vm_name, device_boot_order):
-        new_boot_order = [(self._drive_to_boot_source(device)).path_()
+        new_boot_order = [(self._drive_to_boot_source(device))
                            for device in device_boot_order if device]
 
-        vm = self._lookup_vm_check(vm_name)
-
-        vssd = self._get_vm_setting_data(vm)
+        vssd = self._lookup_vm_check(vm_name)
         old_boot_order = vssd.BootSourceOrder
+
+        # NOTE(abalutoiu): new_boot_order will contain ROOT uppercase
+        # in the device paths while old_boot_order will contain root
+        # lowercase, which will cause the tupple addition result to contain
+        # each device path twice because of the root lowercase and uppercase.
+        # Forcing all the device paths to uppercase fixes the issue.
+        new_boot_order = [x.upper() for x in new_boot_order]
+        old_boot_order = [x.upper() for x in old_boot_order]
         network_boot_devs = set(old_boot_order) ^ set(new_boot_order)
         vssd.BootSourceOrder = tuple(new_boot_order) + tuple(network_boot_devs)
         self._modify_virtual_system(vssd)
+
+    def vm_gen_supports_remotefx(self, vm_gen):
+        """RemoteFX is supported only for generation 1 virtual machines
+        on Windows 8 / Windows Server 2012 and 2012R2.
+
+        :returns: True if the given vm_gen is 1, False otherwise
+        """
+        return vm_gen == constants.VM_GEN_1
+
+    def _validate_remotefx_params(self, monitor_count, max_resolution,
+                                  vram_bytes=None):
+        max_res_value = self._remote_fx_res_map.get(max_resolution)
+        if max_res_value is None:
+            raise exceptions.HyperVRemoteFXException(
+                _("Unsupported RemoteFX resolution: %s") % max_resolution)
+
+        if monitor_count > self._remotefx_max_monitors_map[max_resolution]:
+            raise exceptions.HyperVRemoteFXException(
+                _("Unsuported RemoteFX monitor count: %(count)s for "
+                  "this resolution %(res)s. Hyper-V supports a maximum "
+                  "of %(max_monitors)s monitors for this resolution.")
+                  % {'count': monitor_count,
+                     'res': max_resolution,
+                     'max_monitors': self._remotefx_max_monitors_map[
+                        max_resolution]})
+
+    def _add_3d_display_controller(self, vm, monitor_count,
+                                   max_resolution, vram_bytes=None):
+        synth_3d_disp_ctrl_res = self._get_new_resource_setting_data(
+            self._SYNTH_3D_DISP_CTRL_RES_SUB_TYPE,
+            self._SYNTH_3D_DISP_ALLOCATION_SETTING_DATA_CLASS)
+
+        synth_3d_disp_ctrl_res.MaximumMonitors = monitor_count
+        synth_3d_disp_ctrl_res.MaximumScreenResolution = max_resolution
+
+        self._jobutils.add_virt_resource(synth_3d_disp_ctrl_res, vm)
+
+    def enable_remotefx_video_adapter(self, vm_name, monitor_count,
+                                      max_resolution, vram_bytes=None):
+        vm = self._lookup_vm_check(vm_name, as_vssd=False)
+
+        self._validate_remotefx_params(monitor_count, max_resolution,
+                                       vram_bytes=vram_bytes)
+
+        rasds = _wqlutils.get_element_associated_class(
+            self._conn, self._CIM_RES_ALLOC_SETTING_DATA_CLASS,
+            element_uuid=vm.Name)
+        if [r for r in rasds if r.ResourceSubType ==
+                self._SYNTH_3D_DISP_CTRL_RES_SUB_TYPE]:
+            raise exceptions.HyperVRemoteFXException(
+                _("RemoteFX is already configured for this VM"))
+
+        synth_disp_ctrl_res_list = [r for r in rasds if r.ResourceSubType ==
+                                    self._SYNTH_DISP_CTRL_RES_SUB_TYPE]
+        if synth_disp_ctrl_res_list:
+            self._jobutils.remove_virt_resource(synth_disp_ctrl_res_list[0])
+
+        max_res_value = self._remote_fx_res_map.get(max_resolution)
+        self._add_3d_display_controller(vm, monitor_count, max_res_value,
+                                        vram_bytes)
+        if self._vm_has_s3_controller(vm.ElementName):
+            s3_disp_ctrl_res = [r for r in rasds if r.ResourceSubType ==
+                                self._S3_DISP_CTRL_RES_SUB_TYPE][0]
+            s3_disp_ctrl_res.Address = self._DISP_CTRL_ADDRESS_DX_11
+            self._jobutils.modify_virt_resource(s3_disp_ctrl_res)
+
+    def _vm_has_s3_controller(self, vm_name):
+        return True
